#include "bitmasks.h"

void u8g2_prepare(void)
{
  u8g2.setDisplayRotation(U8G2_R2);
  u8g2.setFont(u8g2_font_6x13_tr);
  u8g2.setFontRefHeightExtendedText();
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setFontDirection(0);
}

void Display_startup_page()
{
  u8g2.setFont(u8g2_font_6x13_tr);
  u8g2.drawXBMP(0, 0, 128, 64, Start_logo_mask);
  u8g2.drawStr(86, 40, "Basil's");
  u8g2.drawStr(74, 50, "Happiness");

  char version_str[7] = "";
  String display_version = "v";
  display_version += BLYNK_FIRMWARE_VERSION;
  display_version.toCharArray(version_str, 7);

  u8g2.setFont(u8g2_font_micro_tr);
  u8g2.drawStr(1, 0, version_str);
}

void Display_Parameters_1_page()
{
  u8g2.setFont(u8g2_font_12x6LED_tr);

  u8g2.drawXBMP(0, 0, 128, 64, Params_1_mask);

  char amb_temperature[5];
  dtostrf(ambient_temp, 4, 1, amb_temperature);
  u8g2.drawStr(33, 10, amb_temperature);

  char amb_humidity[4];
  itoa(ambient_humi, amb_humidity, 10);

  byte humi_pos = 74;
  if (ambient_humi < 100)
    humi_pos = 81;

  u8g2.drawStr(humi_pos, 10, amb_humidity);

  char amb_co2[5];
  itoa(ambient_co2, amb_co2, 10);
  u8g2.drawStr(34, 42, amb_co2);

  char amb_tvoc[5];
  itoa(ambient_tvoc, amb_tvoc, 10);

  byte tvoc_pos = 67;
  if (ambient_tvoc < 1000)
    tvoc_pos = 74;

  u8g2.drawStr(tvoc_pos, 42, amb_tvoc);
}

void Display_Parameters_2_page()
{
  u8g2.setFont(u8g2_font_12x6LED_tr);

  u8g2.drawXBMP(0, 0, 128, 64, Params_2_mask);

  char atm_press[6];
  dtostrf(atm_pressure, 3, 1, atm_press);
  u8g2.drawStr(40, 9, atm_press);

  char wifi_str[6];
  sprintf(wifi_str, "%d%c", wifi_strength, '%');
  u8g2.drawStr(40, 45, wifi_str);

}

void Display_pot_page()
{
  u8g2.setFont(u8g2_font_12x6LED_tr);

  u8g2.drawXBMP(0, 0, 128, 64, Pot_info_mask);

  char soil_sensor1[6];
  itoa(moisture_percentage_sensor1, soil_sensor1, 10);

  byte soil1_pos = 20;
  if (moisture_percentage_sensor1 < 100)
    soil1_pos = 24;

  u8g2.drawStr(soil1_pos, 44, soil_sensor1);

  char soil_sensor2[6];
  itoa(moisture_percentage_sensor2, soil_sensor2, 10);

  byte soil2_pos = 94;
  if (moisture_percentage_sensor2 < 100)
    soil2_pos = 96;

  u8g2.drawStr(soil2_pos, 44, soil_sensor2);

}

void Display_radiator_page()
{
  if (fan_state)
    u8g2.drawXBMP(0, 0, 128, 64, Radiator_On_mask);
  else
    u8g2.drawXBMP(0, 0, 128, 64, Radiator_Off_mask);

  u8g2.setFont(u8g2_font_6x13_tr);
  u8g2.drawStr(8, 0, "Radiator");
  u8g2.drawStr(5, 30, "Fan Speed");

  char radiator_tmp[5];
  dtostrf(radiator_temp, 4, 1, radiator_tmp);
  char fan_spd[4];
  itoa(fan_speed, fan_spd, 10);

  u8g2.setFont(u8g2_font_12x6LED_tr);
  u8g2.drawStr(18, 12, radiator_tmp);

  byte fan_pos = 20;
  if (fan_speed < 100)
    fan_pos = 27;

  u8g2.drawStr(fan_pos, 44, fan_spd);


}

void Display_Processing(byte page)
{
  u8g2.clearBuffer();
  switch (page)
  {
    case 0:
      Display_startup_page();
      break;

    case 1:
      Display_Parameters_1_page();
      break;

    case 2:
      Display_Parameters_2_page();
      break;

    case 3:
      Display_pot_page();
      break;

    case 4:
      Display_radiator_page();
      break;

  };
  u8g2.sendBuffer();
}
