#include <Update.h>
#include <HTTPClient.h>

String overTheAirURL;

void enterOTA(); // Enter OTA Function Prototype

BlynkTimer edgentTimer;


void restartMCU()                                                             // Функция перезагрузки МК
{
  ESP.restart();                                                              // Используя функцию из ESP пакета
  while (1) {};                                                               // Переходя в бесконечный цикл
}

BLYNK_WRITE(InternalPinOTA)                                                   // Функция вызывается, когда на сервере blynk готово ОТА обновление
{
  overTheAirURL = param.asString();                                           // Получаем ссылку на него в строку

  edgentTimer.setTimeout(2000L, []()                                          // Запускаем таймер для таймаута
  {
    Blynk.logEvent("sys_ota", "OTA started");                                 // Отправляем на сервер сообщение, что прошивка началась

    Blynk.disconnect();                                                       // Отключаемся от Blynk, чтобы не мешать прошивке

    enterOTA();                                                               // Переходим в функцию ОТА
  });
}

void enterOTA()                                                               // Функция при прошивке платы по ОТА
{
    while (1)                                                                 // Попадаем в бесконечный цикл
    {
      Serial.print(String("Firmware update URL: ") + overTheAirURL);                 // Выводим адрес ОТА обновления

      HTTPClient http;                                                        // Создаём НТТР клиента
      http.begin(overTheAirURL);                                              // И открываем соединение по адресу ОТА обновления

      int httpCode = http.GET();                                              // Результат запроса
      if (httpCode != HTTP_CODE_OK) {                                         // Если код ответа неудовлетворительный
        Serial.print("HTTP response should be 200");                                 // Пишем в сериал порт
        return;                                                               // Выходим из функции
      }
      int contentLength = http.getSize();                                     // Получаем размер контента
      if (contentLength <= 0) {                                               // Если размер меньше или равен нулю
        Serial.print("Content-Length not defined");                                  // Пишем, что размер не указан
        return;                                                               // Выходим из функции
      }

      bool canBegin = Update.begin(contentLength);                            // Инициализируем обновление, указываем его размер
      if (!canBegin) {                                                        // Если не хватает места для обновления
        Serial.print("Not enough space to begin OTA");                               // Пишем в сериал
        return;                                                               // Выходим из функции
      }

      Client& client = http.getStream();                                      // Начинаем скачивать обновление
      int written = Update.writeStream(client);                               // Начинаем обновление
      if (written != contentLength) {                                         // Если прошили
        Serial.print(String("OTA written ") + written + " / " +                      // Пишем в сериал порт
                     contentLength + " bytes");                                      // Указываем размер обновления, что было прошито
        return;
      }

      if (!Update.end()) {                                                    // Если обновление не было закончено
        Serial.print("Error #" + String(Update.getError()));                         // Выводим код ошибки в сериал порт
        return;                                                               // Выходим из функции
      }

      if (!Update.isFinished()) {                                             // Если не удалось прошить
        Serial.print("Update failed.");                                              // Выводим в сериал порт информацию
        return;                                                               // Выходим из функции
      }

      Serial.print("=== Update successfully completed. Rebooting.");
      restartMCU();                                                           // Перезагрузка микроконтроллера
    }
}
